#!/usr/bin/env python
# -*- coding: utf-8 -*-
import cv2
import numpy as np
import dearpygui.dearpygui as dpg

from openvi.node_editor.util import dpg_get_value, dpg_set_value

from openvi.node.node_abc import DpgNodeABC
from openvi.node_editor.util import convert_cv_to_dpg
from openvi.node.draw_node.draw_util.draw_util import draw_info


class Node(DpgNodeABC):
    _ver = "0.0.1"

    node_label = "Result Image(Large)"
    node_tag = "ResultImageLarge"

    _ratio = 2

    _opencv_setting_dict = None

    def __init__(self):
        pass

    def add_node(
        self,
        parent,
        node_id,
        pos=[0, 0],
        opencv_setting_dict=None,
        callback=None,
    ):
        # タグ名
        tag_node_name = str(node_id) + ":" + self.node_tag
        tag_node_input01_name = (
            tag_node_name + ":" + self.TYPE_IMAGE + ":Input01"
        )
        tag_node_input01_value_name = (
            tag_node_name + ":" + self.TYPE_IMAGE + ":Input01Value"
        )

        # OpenCV向け設定
        self._opencv_setting_dict = opencv_setting_dict
        small_window_w = self._opencv_setting_dict["result_width"]
        small_window_h = self._opencv_setting_dict["result_height"]
        small_window_w *= self._ratio
        small_window_h *= self._ratio

        # 初期化用黒画像
        black_image = np.zeros((small_window_w, small_window_h, 3))
        black_texture = convert_cv_to_dpg(
            black_image,
            small_window_w,
            small_window_h,
        )

        # テクスチャ登録
        with dpg.texture_registry(show=False):
            dpg.add_raw_texture(
                small_window_w,
                small_window_h,
                black_texture,
                tag=tag_node_input01_value_name,
                format=dpg.mvFormat_Float_rgba,
            )

        # ノード
        with dpg.node(
            tag=tag_node_name,
            parent=parent,
            label=self.node_label,
            pos=pos,
        ):
            # 画像
            with dpg.node_attribute(
                tag=tag_node_input01_name,
                attribute_type=dpg.mvNode_Attr_Input,
            ):
                dpg.add_image(tag_node_input01_value_name)

        return tag_node_name

    def update(
        self,
        node_id,
        connection_list,
        node_image_dict,
        node_result_dict,
    ):
        # タグ名
        tag_node_name = str(node_id) + ":" + self.node_tag
        input_value01_tag = (
            tag_node_name + ":" + self.TYPE_IMAGE + ":Input01Value"
        )

        # OpenCV向け設定
        small_window_w = self._opencv_setting_dict["result_width"]
        small_window_h = self._opencv_setting_dict["result_height"]
        draw_info_on_result = self._opencv_setting_dict["draw_info_on_result"]

        # 画像取得元のノード名(ID付き)を取得する
        node_name = ""
        connection_info_src = ""
        for connection_info in connection_list:
            connection_info_src = connection_info[0]
            connection_info_src = connection_info_src.split(":")[:2]
            node_name = connection_info_src[1]
            connection_info_src = ":".join(connection_info_src)

        # 画像取得
        frame = node_image_dict.get(connection_info_src, None)

        # 描画
        if frame is not None:
            if draw_info_on_result and connection_info_src != "":
                node_result = node_result_dict[connection_info_src]
                frame = draw_info(node_name, node_result, frame)
            small_window_w *= self._ratio
            small_window_h *= self._ratio
            texture = convert_cv_to_dpg(
                frame,
                small_window_w,
                small_window_h,
            )
            dpg_set_value(input_value01_tag, texture)

        return frame, None

    def close(self, node_id):
        pass

    def get_setting_dict(self, node_id):
        tag_node_name = str(node_id) + ":" + self.node_tag

        pos = dpg.get_item_pos(tag_node_name)

        setting_dict = {}
        setting_dict["ver"] = self._ver
        setting_dict["pos"] = pos

        return setting_dict

    def set_setting_dict(self, node_id, setting_dict):
        pass
