#!/usr/bin/env python
# -*- coding: utf-8 -*-
import cv2
import numpy as np
import dearpygui.dearpygui as dpg

from openvi.node_editor.util import dpg_get_value, dpg_set_value

from openvi.node.node_abc import DpgNodeABC
from openvi.node_editor.util import convert_cv_to_dpg


class Node(DpgNodeABC):
    _ver = "0.0.1"

    node_label = "Image"
    node_tag = "Image"

    _opencv_setting_dict = None

    _image = {}
    _image_filepath = {}
    _prev_image_filepath = {}

    def __init__(self):
        pass

    def add_node(
        self,
        parent,
        node_id,
        pos=[0, 0],
        opencv_setting_dict=None,
        callback=None,
    ):
        # タグ名
        tag_node_name = str(node_id) + ":" + self.node_tag
        tag_node_input01_name = (
            tag_node_name + ":" + self.TYPE_INT + ":Input01"
        )
        tag_node_output01_name = (
            tag_node_name + ":" + self.TYPE_IMAGE + ":Output01"
        )
        tag_node_output01_value_name = (
            tag_node_name + ":" + self.TYPE_IMAGE + ":Output01Value"
        )

        # OpenCV向け設定
        self._opencv_setting_dict = opencv_setting_dict
        small_window_w = self._opencv_setting_dict["input_window_width"]
        small_window_h = self._opencv_setting_dict["input_window_height"]

        # 初期化用黒画像
        black_image = np.zeros((small_window_w, small_window_h, 3))
        black_texture = convert_cv_to_dpg(
            black_image,
            small_window_w,
            small_window_h,
        )

        # テクスチャ登録
        with dpg.texture_registry(show=False):
            dpg.add_raw_texture(
                small_window_w,
                small_window_h,
                black_texture,
                tag=tag_node_output01_value_name,
                format=dpg.mvFormat_Float_rgba,
            )

        with dpg.file_dialog(
            directory_selector=False,
            show=False,
            modal=True,
            height=int(small_window_h * 3),
            callback=self._callback_file_select,
            id="image_select:" + str(node_id),
        ):
            dpg.add_file_extension(
                "Image (*.bmp *.jpg *.png *.gif){.bmp,.jpg,.png,.gif}"
            )
            dpg.add_file_extension("", color=(150, 255, 150, 255))

        # ノード
        with dpg.node(
            tag=tag_node_name,
            parent=parent,
            label=self.node_label,
            pos=pos,
        ):
            # ファイル選択
            with dpg.node_attribute(
                tag=tag_node_input01_name,
                attribute_type=dpg.mvNode_Attr_Static,
            ):
                dpg.add_button(
                    label="Select Image",
                    width=small_window_w,
                    callback=lambda: dpg.show_item(
                        "image_select:" + str(node_id),
                    ),
                )
            # カメラ画像
            with dpg.node_attribute(
                tag=tag_node_output01_name,
                attribute_type=dpg.mvNode_Attr_Output,
            ):
                dpg.add_image(tag_node_output01_value_name)

        return tag_node_name

    def update(
        self,
        node_id,
        connection_list,
        node_image_dict,
        node_result_dict,
    ):
        tag_node_name = str(node_id) + ":" + self.node_tag
        output_value01_tag = (
            tag_node_name + ":" + self.TYPE_IMAGE + ":Output01Value"
        )

        small_window_w = self._opencv_setting_dict["input_window_width"]
        small_window_h = self._opencv_setting_dict["input_window_height"]

        # VideoCapture()インスタンス生成
        image_path = self._image_filepath.get(str(node_id), None)
        prev_image_path = self._prev_image_filepath.get(str(node_id), None)
        if prev_image_path != image_path:
            self._image[str(node_id)] = cv2.imread(image_path)
            self._prev_image_filepath[str(node_id)] = image_path

        # 画像取得
        frame = self._image.get(str(node_id), None)

        # 描画
        if frame is not None:
            texture = convert_cv_to_dpg(
                frame,
                small_window_w,
                small_window_h,
            )
            dpg_set_value(output_value01_tag, texture)

        return frame, None

    def close(self, node_id):
        pass

    def get_setting_dict(self, node_id):
        tag_node_name = str(node_id) + ":" + self.node_tag

        pos = dpg.get_item_pos(tag_node_name)

        setting_dict = {}
        setting_dict["ver"] = self._ver
        setting_dict["pos"] = pos

        return setting_dict

    def set_setting_dict(self, node_id, setting_dict):
        pass

    def _callback_file_select(self, sender, data):
        if data["file_name"] != ".":
            node_id = sender.split(":")[1]
            self._image_filepath[node_id] = data["file_path_name"]
